@@ -1,29 +1,19 @@
 use crate::common::*;
 
-#[derive(Debug, PartialEq)]
+use serde::Serialize;
+
+#[derive(Debug, PartialEq, Serialize)]
 pub(crate) struct Justfile<'src> {
-  pub(crate) recipes: Table<'src, Rc<Recipe<'src>>>,
-  pub(crate) assignments: Table<'src, Assignment<'src>>,
   pub(crate) aliases: Table<'src, Alias<'src>>,
+  pub(crate) assignments: Table<'src, Assignment<'src>>,
+  #[serde(serialize_with = "keyed::serialize_option")]
+  pub(crate) first: Option<Rc<Recipe<'src>>>,
+  pub(crate) recipes: Table<'src, Rc<Recipe<'src>>>,
   pub(crate) settings: Settings<'src>,
   pub(crate) warnings: Vec<Warning>,
 }
 
 impl<'src> Justfile<'src> {
-  pub(crate) fn first(&self) -> Option<&Recipe<'src>> {
-    let mut first: Option<&Recipe<Dependency>> = None;
-    for recipe in self.recipes.values() {
-      if let Some(first_recipe) = first {
-        if recipe.line_number() < first_recipe.line_number() {
-          first = Some(recipe);
-        }
-      } else {
-        first = Some(recipe);
-      }
-    }
-    first
-  }
-
   pub(crate) fn count(&self) -> usize {
     self.recipes.len()
   }
@@ -84,10 +74,6 @@ impl<'src> Justfile<'src> {
     overrides: &BTreeMap<String, String>,
     arguments: &[String],
   ) -> RunResult<'src, ()> {
-    if let Err(error) = InterruptHandler::install(config.verbosity) {
-      warn!("Failed to set CTRL-C handler: {}", error);
-    }
-
     let unknown_overrides = overrides
       .keys()
       .filter(|name| !self.assignments.contains_key(name.as_str()))
@@ -101,7 +87,7 @@ impl<'src> Justfile<'src> {
     }
 
     let dotenv = if config.load_dotenv {
-      load_dotenv(&config, &self.settings, &search.working_directory)?
+      load_dotenv(config, &self.settings, &search.working_directory)?
     } else {
       BTreeMap::new()
     };
@@ -139,7 +125,7 @@ impl<'src> Justfile<'src> {
         binary, arguments, ..
       } => {
         let mut command = if config.shell_command {
-          let mut command = self.settings.shell_command(&config);
+          let mut command = self.settings.shell_command(config);
           command.arg(binary);
           command
         } else {
@@ -178,7 +164,7 @@ impl<'src> Justfile<'src> {
             print!("{}", value);
           } else {
             return Err(Error::EvalUnknownVariable {
-              suggestion: self.suggest_variable(&variable),
+              suggestion: self.suggest_variable(variable),
               variable: variable.clone(),
             });
           }
@@ -206,7 +192,7 @@ impl<'src> Justfile<'src> {
 
     let argvec: Vec<&str> = if !arguments.is_empty() {
       arguments.iter().map(String::as_str).collect()
-    } else if let Some(recipe) = self.first() {
+    } else if let Some(recipe) = &self.first {
       let min_arguments = recipe.min_arguments();
       if min_arguments > 0 {
         return Err(Error::DefaultRecipeRequiresArguments {
@@ -271,7 +257,7 @@ impl<'src> Justfile<'src> {
 
     let mut ran = BTreeSet::new();
     for (recipe, arguments) in grouped {
-      self.run_recipe(&context, recipe, arguments, &dotenv, &search, &mut ran)?;
+      self.run_recipe(&context, recipe, arguments, &dotenv, search, &mut ran)?;
     }
 
     Ok(())
@@ -354,8 +340,8 @@ impl<'src> Justfile<'src> {
     }
 
     let mut invocation = vec![recipe.name().to_owned()];
-    for argument in arguments.iter().cloned() {
-      invocation.push(argument.to_owned());
+    for argument in arguments {
+      invocation.push((*argument).to_string());
     }
 
     ran.insert(invocation);
